//-----------------------------------------------------------
//  the basic Button Class.
//  Buttons are Instances of HUDOverlay so they can use their own Renderfunction.
//  Provides Button Graphics and basic MouseOver, LeftBtn and RightBtn functions.
//  checks if Mouse is Over and sets Mouse.selectedButton once LeftBtn is clicked
//
//  Copyright 2005 Friedrich Kirschner
//-----------------------------------------------------------
class MSBGuiOverlay extends HUDOverlay
config (MSBCode);

#exec texture Import File=Textures\Node.tga Name=Node_Icon Mips=Off Alpha=1

var config float xPos,               //position and area on screen
                 yPos,
                 width,
                 height;

var int u,v;                         // Icon Resolution

var color col,textCol,tooltipCol,overlayCol;                       //Button Color,Tooltip color,MouseOver color

var MSBMouseOverlay Mouse;            //Pointer to Mouse
var MSBController M;                  //Pointer to PlayerController

var bool bOver;
var bool hasClicked;                 // used for Two-way Buttons in Cinjunction with bHasClickedAgain
var bool rHasClicked;
var() int SubLevel;                    // indicates the Buttons Menu "deepness".
var bool bEnabled;
var bool bMainMenu;
var() bool bPermanent;                 // Left on screen when playing

var Material Mat;                    //Button Icon
var string tooltip;                  //Button Tooltip
var string BtnName;

var string FontNameSmall;
var string FontNameMedium;
var string FontNameBig;

var config float tooltipFontScale;
var config float buttonFontScale;
var config float fontScale;

event PostBeginPlay()                //establish Pointer to PlayerController and define Screenarea
{
super.PostBeginPlay();
M=MSBController(Level.GetLocalPlayerController());
if (Mat!=None)
    {
    u=Mat.MaterialUSize();
    v=Mat.MaterialVSize();
    }

if (M.myHUD!=None)
  sortButtonList();
}

function sortButtonList()
{
local Hud h;
local MSBGuiOverlay mOverlay;

h=M.myHUD;
//first pop the mouse
mOverlay=MSBGuiOverlay(h.Overlays[h.overlays.length-1]);
h.RemoveHudOverlay(mOverlay);
//connect the Mouse
Mouse=MSBMouseOverlay(mOverlay);
//then add ouselves
h.AddHudOverlay(self);
// and PUSH the mouse again
h.AddHudOverlay(mOverlay);
}


function Render(Canvas C)           //display Button and call MouseOver, LeftBtn, RightBtn
{
C.Font=Font(DynamicLoadObject(FontNameSmall, class'Font'));
C.FontScaleX=fontScale;
C.FontScaleY=fontScale;
initButton(C);
bOver=checkover(Mouse.xpos,Mouse.ypos);
if (bOver)
  Mouse.hudTargetActor=self;     //pointer from Mouse to Button it hovers on


if (bEnabled)
  {
  if (bOver)
    MouseOver(C);
  if (Mouse.btnOne && !hasclicked && bOver)
     {
      hasClicked=true;
      LeftBtn(C);
     }
  if (Mouse.btnTwo && !rHasclicked && bOver)
    {
     rHasClicked=true;
     RightBtn(C);
    }
  }
}

function bool checkOver(float X, float Y)  //check if Mouse is over Button
{
if (x>xPos && x<xPos+width)
  if (y>yPos && y<yPos+height)
    return true;
return false;
}

function initButton(Canvas C)             //draw the Button on Screen
{
C.SetDrawColor(col.R,col.G, col.B, col.A);
C.SetPos(xPos,yPos);
if (Mat!=None)
    C.DrawTile(Mat,width,height,0,0,U,V);
C.SetPos(xPos,yPos);
}

function LeftBtn(Canvas C)
{
local int i;
// destroy other subMenus at same level
for (i=0;i<M.myHUD.Overlays.length;i++)
  {
  if (M.myHUD.Overlays[i]!=self)  //don't deselect self!!!
    MSBGuiOverlay(M.myHUD.Overlays[i]).deselect();
  if (MSBGuiOverlay(M.myHUD.Overlays[i]).SubLevel>Sublevel)
    M.myHUD.Overlays[i--].Destroy();
  }
Mouse.btnOne=false;
}

function RightBtn(Canvas C)
{
Mouse.btnTwo=false;
}

function MouseOver(Canvas C)
{
C.SetDrawColor(overlayCol.R,overlayCol.G, overlayCol.B, overlayCol.A);
C.SetPos(xPos,yPos);
if (Mat!=None)
  C.DrawTile(Mat,width,height,0,0,U,V);
C.SetPos(xPos+width,yPos+12);
C.SetDrawColor(tooltipCol.R,tooltipCol.G, tooltipCol.B, tooltipCol.A);
C.FontScaleX=tooltipFontScale;
C.FontScaleY=tooltipFontScale;
C.DrawText(tooltip);
C.FontScaleX=FontScale;
C.FontScaleY=FontScale;
}

function deSelect()
{
hasClicked=false;
rHasClicked=false;
bEnabled=true;
}

defaultproperties
{
FontNameSmall="MovieSandboxTextures.Font.MSBSmallFont"
//FontNameSmall="Engine.DefaultFont"
FontNameMedium="MovieSandboxTextures.Font.MSBMediumFont"
FontNameBig="MovieSandboxTextures.Font.MSBLargeFont"
bEnabled=true
tooltip="Your tip here!"
Mat=Material'Node_Icon'
xpos=100
ypos=100
width=24
height=24
col=(R=255,G=255,B=255,A=255)
TextCol=(R=255,G=255,B=255,A=255)
tooltipCol=(R=255,G=255,B=255,A=255)
overlayCol=(R=255,G=0,B=0,A=100)

fontScale=1
buttonFontScale=1
tooltipFontScale=1
}
